@@ -138,6 +138,20 @@ def only_ssl(fn):
 
     return decorated
 
+def only_telnet(fn):
+    import nose.plugins.skip
+    import pycurl
+
+    @functools.wraps(fn)
+    def decorated(*args, **kwargs):
+        # pycurl.version_info()[8] is a tuple of protocols supported by libcurl
+        if 'telnet' not in pycurl.version_info()[8]:
+            raise nose.plugins.skip.SkipTest('libcurl does not support telnet')
+
+        return fn(*args, **kwargs)
+
+    return decorated
+
 def only_ssl_backends(*backends):
     def decorator(fn):
         import nose.plugins.skip
